unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, ExtCtrls, EditPlugAPI, EditPlugAPITypes;

type
  TForm1 = class(TForm)
    Panel2: TPanel;
    WinPan: TPanel;
    Button1: TButton;
    Popup: TPopupMenu;
    bUnlock: TButton;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PopupItemClick(Sender: TObject);
    procedure PopupPopup(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure WinPanResize(Sender: TObject);
    procedure bUnlockClick(Sender: TObject);
  private
    { Private declarations }
    FirstShow: boolean;
    WinToLock: HWND;
    procedure UnlockCurrentWin;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  gPlugId: HINST;

implementation

{$R *.DFM}

uses stringutils;

{----- EXPORTS ----------------------------------------------------------------}

procedure plgSetId(id: HINST); cdecl;
begin
  gPlugId:= id;
end; exports plgSetId;

function plgShow: HWND; cdecl;
begin
  if not assigned(form1) then begin
    form1:= tform1.create(nil);
    form1.show;
    epCreatePluginTab(gPlugId, form1.handle);
  end else begin
    form1.show;
  end;
  result:= form1.Handle;
end; exports plgShow;

procedure plgWinFree; cdecl;
begin
  if assigned(form1) then begin
    // return current locked window to taskbar/desktop
    form1.UnlockCurrentWin;
    form1.Free;
    form1:=nil;
  end;
end; exports plgWinFree;

function plgGetWinHandle: HWND; cdecl;
begin
  result:=0;
  if assigned(form1) then result:= form1.Handle;
end; exports plgGetWinHandle;

{------------------------------------------------------------------------------}

function GetText(Wnd: HWND): string;
var
  textlength: Integer;
  Text: PChar;
begin
  textlength := SendMessage(Wnd, WM_GETTEXTLENGTH, 0, 0);
  if textlength = 0 then Result := ''
  else
  begin
    GetMem(Text, textlength + 1);
    SendMessage(Wnd, WM_GETTEXT, textlength + 1, Integer(Text));
    Result := Text;
    FreeMem(Text);
  end;
end;

function EnumWindowsProc(Wnd: HWND; lParam: lParam): BOOL; stdcall;
var
  mi: TMenuItem;
  txt: string;
begin
  Result := True;
  if (IsWindowVisible(Wnd) or IsIconic(wnd)) and
    ((GetWindowLong(Wnd, GWL_HWNDPARENT) = 0) or
    (GetWindowLong(Wnd, GWL_HWNDPARENT) = GetDesktopWindow)) and
    (GetWindowLong(Wnd, GWL_EXSTYLE) and WS_EX_TOOLWINDOW = 0) then
  begin
    txt := GetText(Wnd);
    if (wnd <> epGetMainWinHandle)
      and (lowercase(LeftStr(txt, length('editplug'))) <> 'editplug' )then
    begin
      mi := TMenuItem.create(form1.popup);
      mi.caption:= txt + ' - Handle: ' + IntToStr(Wnd);
      mi.OnClick := form1.PopupItemClick;
      form1.popup.Items.Add(mi);
    end;
  end;
end;


procedure TForm1.FormShow(Sender: TObject);
begin
  if FirstShow = true then begin
    FirstShow := false;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FirstShow := true;
end;

function FindHandle(s: string): HWND;
var
  found, i: integer;
  rslt: string;
const
  findstr = ' - Handle: ';
begin
  result := 0;
  rslt := '';
  found := pos(findstr, s);
  if found > 0 then begin
    for i := found + length(findstr) to length(s) do rslt := rslt + s[i];
    result := strtoint(rslt);
  end;
end;

// set old locked window to the taskbar/desktop again
procedure TForm1.UnlockCurrentWin;
begin
  if WinToLock <> 0 then windows.setparent(WinToLock, 0);
end;

procedure TForm1.PopupItemClick(Sender: TObject);
var
  h: HWND;
  mi: TMenuItem;
begin
  if sender is TMenuItem then begin
    mi := sender as TMenuItem;
    h := FindHandle(mi.caption);
    UnlockCurrentWin;
    WinToLock := h;
    windows.setparent(WinToLock, WinPan.handle);
    MoveWindow(WinToLock, 0, 0, WinPan.Width, WinPan.Height, true);
  end;
end;

procedure TForm1.PopupPopup(Sender: TObject);
var
  Param: Longint;
begin
  popup.Items.Clear;
  EnumWindows(@EnumWindowsProc, Param);
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  pnt: TPoint;
begin
  if GetCursorPos(pnt) then
    Popup.Popup(pnt.X, pnt.Y);
end;

procedure TForm1.WinPanResize(Sender: TObject);
begin
  if WinToLock <> 0 then MoveWindow(WinToLock, 0, 0, WinPan.Width, WinPan.Height, true);
end;

procedure TForm1.bUnlockClick(Sender: TObject);
begin
  UnlockCurrentWin;
  WinToLock := 0;
end;

initialization

finalization

end.
